@@ -1,5 +1,5 @@
 # JUBE Benchmarking Environment
-# Copyright (C) 2008-2019
+# Copyright (C) 2008-2021
 # Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre
 # http://www.fz-juelich.de/jsc/jube
 #
@@ -97,9 +97,10 @@ class Table(KeyValuesResult):
                 output = "{0}:\n".format(self.name)
             else:
                 output = ""
+
             output += jube2.util.output.text_table(
                 data, use_header_line=True, auto_linebreak=False, colw=colw,
-                indent=0, pretty=(self._style == "pretty"),
+                indent=0, style=self._style,
                 separator=self._separator, transpose=self._transpose)
 
             return output
